@@ -6,7 +6,7 @@ import databroker
 import event_model
 import matplotlib.pyplot as plt
 import numpy as np
-from bluesky.callbacks.broker import LiveImage
+from bluesky.callbacks.core import make_class_safe
 from bluesky.callbacks.stream import LiveDispatcher
 from databroker.v2 import Broker
 from event_model import RunRouter
@@ -20,7 +20,8 @@ import pdfstream.callbacks.from_descriptor as from_desc
 import pdfstream.callbacks.from_event as from_event
 import pdfstream.callbacks.from_start as from_start
 import pdfstream.integration.tools as integ
-from pdfstream.callbacks.basic import LiveMaskedImage, LiveWaterfall, NumpyExporter, SmartScalarPlot
+from pdfstream.callbacks.basic import MyLiveImage, LiveMaskedImage, LiveWaterfall, DataFrameExporter, \
+    SmartScalarPlot
 from pdfstream.units import LABELS
 from pdfstream.vend.formatters import SpecialStr
 
@@ -133,7 +134,7 @@ class AnalysisStream(LiveDispatcher):
     """
 
     def __init__(self, config: AnalysisConfig):
-        super().__init__()
+        super(AnalysisStream, self).__init__()
         self.config = config
         self.cache = {}
         self.db = config.raw_db
@@ -152,7 +153,8 @@ class AnalysisStream(LiveDispatcher):
             default_composition="Ni"
         )
         self.cache["indeps"] = from_start.get_indeps(doc, exclude={"time"})
-        super().start(dict(**doc, an_config=self.config.to_dict()))
+        new_start = dict(**doc, an_config=self.config.to_dict())
+        super(AnalysisStream, self).start(new_start)
 
     def event_page(self, doc):
         for event_doc in event_model.unpack_event_page(doc):
@@ -174,7 +176,7 @@ class AnalysisStream(LiveDispatcher):
             dk_id_key=self.config.dk_id_key,
             db=self.db
         )
-        super().descriptor(doc)
+        super(AnalysisStream, self).descriptor(doc)
 
     def event(self, doc, _md=None):
         # the raw image in the data
@@ -200,7 +202,7 @@ class AnalysisStream(LiveDispatcher):
         self.process_event(EventDoc(data=data, descriptor=doc["descriptor"]))
 
     def stop(self, doc, _md=None):
-        super().stop(doc)
+        super(AnalysisStream, self).stop(doc)
 
 
 def process(
@@ -322,6 +324,7 @@ class ExportConfig(BasicExportConfig):
         return {"file_prefix": SpecialStr(section.get("file_prefix"))}
 
 
+@make_class_safe
 class Exporter(RunRouter):
     """Export the processed data to file systems, including."""
 
@@ -335,38 +338,38 @@ class ExporterFactory:
 
     def __init__(self, config: ExportConfig):
         self.config = config
+
+    def __call__(self, name: str, doc: dict) -> tp.Tuple[list, list]:
+        if name != "start":
+            return [], []
         tiff_base = self.config.tiff_base
         tiff_base.mkdir(exist_ok=True, parents=True)
-        self.callbacks = []
+        callbacks = []
         if self.config.tiff_setting is not None:
             cb = TiffSerializer(
                 str(tiff_base.joinpath("images")),
                 **self.config.tiff_setting
             )
-            self.callbacks.append(cb)
+            callbacks.append(cb)
         if self.config.json_setting is not None:
             cb = JsonSerializer(
                 str(tiff_base.joinpath("metadata")),
                 **self.config.json_setting
             )
-            self.callbacks.append(cb)
+            callbacks.append(cb)
         if self.config.csv_setting is not None:
             cb = CSVSerializer(
-                str(tiff_base.joinpath("datasheets")),
+                str(tiff_base.joinpath("scalar_data")),
                 **self.config.csv_setting
             )
-            self.callbacks.append(cb)
-        if self.config.csv_setting is not None:
-            cb = NumpyExporter(
-                str(tiff_base.joinpath("arrays")),
+            callbacks.append(cb)
+        if self.config.npy_setting is not None:
+            cb = DataFrameExporter(
+                str(tiff_base.joinpath("array_data")),
                 **self.config.npy_setting
             )
-            self.callbacks.append(cb)
-
-    def __call__(self, name: str, doc: dict) -> tp.Tuple[list, list]:
-        if name != "start":
-            return [], []
-        return self.callbacks, []
+            callbacks.append(cb)
+        return callbacks, []
 
 
 class VisConfig(ConfigParser):
@@ -476,6 +479,7 @@ class VisConfig(ConfigParser):
         return {"ylabel": section.get("ylabel", fallback=LABELS.chi[0])}
 
 
+@make_class_safe
 class Visualizer(RunRouter):
     """Visualize the analyzed data. It can be subscribed to a live dispatcher."""
 
@@ -485,14 +489,7 @@ class Visualizer(RunRouter):
 
     def show_figs(self):
         """Show all the figures in the callbacks in the factory."""
-        for cb in self._factory.cb_lst:
-            if isinstance(cb, (LiveImage, LiveMaskedImage)):
-                cb.cs._fig.show()
-            elif isinstance(cb, LiveWaterfall):
-                cb.show()
-            elif isinstance(cb, SmartScalarPlot):
-                cb.show()
-        return
+        return self._factory.show()
 
 
 class VisFactory:
@@ -504,11 +501,11 @@ class VisFactory:
         # images
         if self.config.vis_dk_sub_image is not None:
             self.cb_lst.append(
-                LiveImage("dk_sub_image", **self.config.vis_dk_sub_image)
+                MyLiveImage("dk_sub_image", **self.config.vis_dk_sub_image)
             )
         if self.config.vis_bg_sub_image is not None:
             self.cb_lst.append(
-                LiveImage("bg_sub_image", **self.config.vis_dk_sub_image)
+                MyLiveImage("bg_sub_image", **self.config.vis_dk_sub_image)
             )
         if self.config.vis_masked_image is not None:
             self.cb_lst.append(
@@ -530,17 +527,23 @@ class VisFactory:
                 fig.show()
 
         # scalar data
-        fig1 = plt.figure()
-        axes1 = (fig1.add_subplot(grid) for grid in GridSpec(2, 2))
-        for field, vis_config in [
+        fields_and_configs = [
             ("chi_max", self.config.vis_chi_max),
             ("chi_argmax", self.config.vis_chi_argmax),
             ("gr_max", self.config.vis_gr_max),
             ("gr_argmax", self.config.vis_gr_argmax)
-        ]:
+        ]
+        # make a figure of stacking axes
+        fig1 = plt.figure()
+        axes1 = [fig1.add_subplot(grid) for grid in GridSpec(len(fields_and_configs), 1, hspace=0)]
+        if len(axes1) > 1:
+            for ax in axes1[:-1]:
+                ax.get_xaxis().set_visible(False)
+        # link the axes with callbacks
+        for (field, vis_config), ax in zip(fields_and_configs, axes1):
             if vis_config is not None:
                 self.cb_lst.append(
-                    SmartScalarPlot(field, ax=next(axes1), **vis_config, marker="o")
+                    SmartScalarPlot(field, ax=ax, **vis_config, marker="o")
                 )
         fig1.show()
 
@@ -548,3 +551,11 @@ class VisFactory:
         if name != "start":
             return [], []
         return self.cb_lst, []
+
+    def show(self):
+        for cb in self.cb_lst:
+            try:
+                cb.show()
+            except AttributeError:
+                pass
+        return
