@@ -1,8 +1,8 @@
 import typing as tp
-from datetime import datetime
 from pathlib import Path
 
 import numpy as np
+import pandas as pd
 from bluesky.callbacks import CallbackBase
 from bluesky.callbacks.best_effort import LivePlot, LiveScatter
 from bluesky.callbacks.broker import LiveImage
@@ -15,60 +15,86 @@ from xpdview.waterfall import Waterfall
 
 import pdfstream.callbacks.from_descriptor as fd
 import pdfstream.callbacks.from_start as fs
+from pdfstream.vend.formatters import SpecialStr
 
 
-class StartStopCallback(CallbackBase):
-    """Print the time for analysis"""
-
-    def __init__(self):
-        super(StartStopCallback, self).__init__()
-
-    def start(self, doc):
-        print("[{}] Receive the start of run {}".format(datetime.now(), doc["uid"]))
-
-    def stop(self, doc):
-        print("[{}] Receive the stop of run {}".format(datetime.now(), doc.get("run_start", "NA")))
-
-
-class NumpyExporter(CallbackBase):
-    """An exporter to export the 1d array data in .npy file."""
+class ArrayExporter(CallbackBase):
+    """An base class for the callbacks to find and export the 1d array."""
+    file_suffix = ""
+    file_stem = "{descriptor[name]}-{field}-{event[seq_num]}"
 
     def __init__(self, directory: str, *, file_prefix: str, data_keys: tp.List[str] = None):
-        super(NumpyExporter, self).__init__()
+        super(ArrayExporter, self).__init__()
         self.directory = Path(directory)
         self.directory.mkdir(parents=True, exist_ok=True)
-        self.file_template = file_prefix + "{data_key}-{event[seq_num]}.npy"
+        self.file_template = SpecialStr(
+            file_prefix + self.file_stem + self.file_suffix
+        )
         self.data_keys = data_keys
-        self.cache = dict()
+        self.start_doc = None
+        self.descriptor_doc = None
 
     def start(self, doc):
-        self.cache = dict()
-        self.cache["start"] = doc
-        super(NumpyExporter, self).start(doc)
+        self.start_doc = doc
+        super(ArrayExporter, self).start(doc)
 
     def descriptor(self, doc):
         if not self.data_keys:
             self.data_keys = list(fd.yield_1d_array(doc["data_keys"]))
-        super(NumpyExporter, self).descriptor(doc)
+        self.descriptor_doc = doc
+        super(ArrayExporter, self).descriptor(doc)
+
+    def event(self, doc):
+        self.export(doc)
+        super(ArrayExporter, self).event(doc)
 
     def event_page(self, doc):
         for event in unpack_event_page(doc):
             self.event(event)
 
-    def event(self, doc):
+    def stop(self, doc):
+        self.start_doc = None
+        self.descriptor_doc = None
+        super(ArrayExporter, self).stop(doc)
+
+    def export(self, doc):
+        pass
+
+
+class NumpyExporter(ArrayExporter):
+    """An exporter to export the 1d array data in .npy file."""
+    file_suffix = ".npy"
+
+    def export(self, doc):
         for data_key in self.data_keys:
             arr: np.ndarray = doc["data"][data_key]
-            filename = self.file_template.format(start=self.cache["start"], event=doc, data_key=data_key)
+            filename = self.file_template.format(start=self.start_doc, descriptor=self.descriptor_doc, event=doc,
+                                                 field=data_key)
             filepath = self.directory.joinpath(filename)
             np.save(str(filepath), arr)
-        super(NumpyExporter, self).event(doc)
-
-    def stop(self, doc):
-        self.cache = dict()
-        super(NumpyExporter, self).stop(doc)
 
 
-class LiveMaskedImage(LiveImage):
+class DataFrameExporter(ArrayExporter):
+    """An exporter to export data in a dataframe in the .csv file."""
+    file_suffix = ".csv"
+
+    def export(self, doc):
+        _data = {data_key: pd.Series(doc["data"][data_key]) for data_key in self.data_keys}
+        df = pd.DataFrame(data=_data)
+        filename = self.file_template.format(start=self.start_doc, descriptor=self.descriptor_doc, event=doc,
+                                             field="data")
+        filepath = self.directory.joinpath(filename)
+        df.to_csv(str(filepath))
+
+
+class MyLiveImage(LiveImage):
+    """A customized LiveImage."""
+
+    def show(self):
+        self.cs._fig.show()
+
+
+class LiveMaskedImage(MyLiveImage):
     """Live image show of a image with a mask."""
 
     def __init__(self, field: str, msk_field: str, *, cmap: str, norm: tp.Callable = None,
@@ -107,7 +133,7 @@ class MyWaterfall(Waterfall):
         self.ydist = 0
         self.xdist = 0
 
-        y_offset_slider_ax = self.fig.add_axes([0.15, 0.95, 0.3, 0.035])
+        y_offset_slider_ax = self.fig.add_axes([0.15, 0.95, 0.25, 0.035])
         self.y_offset_slider = Slider(
             y_offset_slider_ax,
             "y-offset",
@@ -118,7 +144,7 @@ class MyWaterfall(Waterfall):
         )
         self.y_offset_slider.on_changed(self.update_y_offset)
 
-        x_offset_slider_ax = self.fig.add_axes([0.6, 0.95, 0.3, 0.035])
+        x_offset_slider_ax = self.fig.add_axes([0.6, 0.95, 0.25, 0.035])
         self.x_offset_slider = Slider(
             x_offset_slider_ax,
             "x-offset",
