@@ -1,23 +1,30 @@
 """The functions used in the command line interface. The input and output are all files."""
 import typing as tp
+from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor, as_completed
 from pathlib import Path, PurePath
 
 import matplotlib.pyplot as plt
+import numpy as np
 from matplotlib.axes import Axes
-from pkg_resources import resource_filename
 
-import pdfstream.calibration as calib
 import pdfstream.integration as integ
 import pdfstream.io as io
-import pdfstream.modeling as model
 import pdfstream.visualization as vis
 
 
 def integrate(
-    poni_file: str, *img_files: str, bg_img_file: str = None,
-    output_dir: str = ".", bg_scale: float = None, mask_setting: tp.Union[dict, str] = None,
-    integ_setting: dict = None, plot_setting: tp.Union[dict, str] = None,
-    img_setting: tp.Union[dict, str] = None
+    poni_file: str,
+    *img_files: str,
+    bg_img_file: str = None,
+    mask_file: str = None,
+    output_dir: str = ".",
+    bg_scale: float = None,
+    mask_setting: tp.Union[dict, str] = None,
+    integ_setting: dict = None,
+    plot_setting: tp.Union[dict, str] = None,
+    img_setting: tp.Union[dict, str] = None,
+    parallel: bool = False,
+    test: bool = False
 ) -> tp.List[str]:
     """Conduct azimuthal integration on the two dimensional diffraction images.
 
@@ -39,6 +46,9 @@ def integrate(
         The path to the background image file. It should have the same dimension as the data image. If None,
         no background subtraction will be done.
 
+    mask_file : str
+        The .npy file of the mask array. Use pyFai convention. 1 are masked pixels, 0 are good pixels.
+
     output_dir : str
         The directory to save the chi data file. Default current working directory.
 
@@ -65,25 +75,68 @@ def integrate(
         'z_score', which determines the range of the colormap. The range is mean +/- z_score * std in the
         statistics of the image. To turn of the image, enter "OFF".
 
+    parallel : bool
+        If True, run the processing in multiple process.
+
+    test : bool
+        If True, run in test mode (for developers).
+
     Returns
     -------
     chi_files : a list of strings
         The path to the output chi file.
     """
+    Path(output_dir).mkdir(parents=True, exist_ok=True)
+    if parallel:
+        executor = ProcessPoolExecutor() if not test else ThreadPoolExecutor()
+        jobs = [
+            executor.submit(
+                _integrate, poni_file, img_file, bg_img_file=bg_img_file, mask_file=mask_file,
+                output_dir=output_dir, bg_scale=bg_scale, mask_setting=mask_setting, integ_setting=integ_setting,
+                plot_setting=plot_setting, img_setting=img_setting, test=test
+            )
+            for img_file in img_files
+        ]
+        return [job.result() for job in as_completed(jobs)]
+    return [
+        _integrate(
+            poni_file, img_file, bg_img_file=bg_img_file, mask_file=mask_file,
+            output_dir=output_dir, bg_scale=bg_scale, mask_setting=mask_setting, integ_setting=integ_setting,
+            img_setting=img_setting, test=test
+        )
+        for img_file in img_files
+    ]
+
+
+def _integrate(
+    poni_file: str,
+    img_file: str,
+    bg_img_file: str = None,
+    mask_file: str = None,
+    output_dir: str = ".",
+    bg_scale: float = None,
+    mask_setting: tp.Union[dict, str] = None,
+    integ_setting: dict = None, plot_setting: tp.Union[dict, str] = None,
+    img_setting: tp.Union[dict, str] = None,
+    test: bool = False
+) -> str:
+    """Sub-function for integrate."""
     if integ_setting is None:
         integ_setting = dict()
     ai = io.load_ai_from_poni_file(poni_file)
     bg_img = io.load_img(bg_img_file) if bg_img_file else None
-    chi_paths = []
-    for img_file in img_files:
-        img = io.load_img(img_file)
-        chi_name = Path(img_file).with_suffix('.chi').name
-        chi_path = Path(output_dir).joinpath(chi_name)
-        integ_setting.update({'filename': str(chi_path)})
-        integ.get_chi(ai, img, bg_img, bg_scale=bg_scale, mask_setting=mask_setting,
-                      integ_setting=integ_setting, plot_setting=plot_setting, img_setting=img_setting)
-        chi_paths.append(str(chi_path))
-    return chi_paths
+    mask = np.load(mask_file) if mask_file else None
+    img = io.load_img(img_file)
+    chi_name = Path(img_file).with_suffix('.chi').name
+    chi_path = Path(output_dir).joinpath(chi_name)
+    integ_setting.update({'filename': str(chi_path)})
+    integ.get_chi(
+        ai, img, bg_img=bg_img, mask=mask, bg_scale=bg_scale, mask_setting=mask_setting,
+        integ_setting=integ_setting, plot_setting=plot_setting, img_setting=img_setting,
+    )
+    if not test:
+        plt.show()
+    return chi_path
 
 
 def average(out_file: str, *img_files, weights: tp.List[float] = None) -> None:
@@ -194,7 +247,7 @@ def waterfall(
         label=label, minor_tick=minor_tick, legends=legends, colors=colors, **kwargs
     )
     if show_fig:
-        plt.show(block=False)
+        plt.show()
     return ax
 
 
@@ -273,103 +326,5 @@ def visualize(
         data, ax=ax, mode=mode, normal=normal, text=text, text_xy=text_xy, label=label,
         minor_tick=minor_tick, legends=legends, color=color, **kwargs)
     if show_fig:
-        plt.show(block=False)
+        plt.show()
     return ax
-
-
-def instrucalib(
-    poni_file: str, img_file: str, cfg_file: str = None, stru_file: str = None, output_dir=".",
-    fit_range: tp.Tuple[float, float, float] = (2.0, 60.0, 0.01),
-    qdamp0: float = 0.04, qbroad0: float = 0.02,
-    bg_img_file: str = None, bg_scale: float = None,
-    mask_setting: tp.Union[dict, str] = None, integ_setting: dict = None,
-    chi_plot_setting: tp.Union[dict, str] = None, img_setting: tp.Union[dict, str] = None,
-    pdf_plot_setting: tp.Union[dict, str] = None, ncpu: int = None
-):
-    """Calibrate the 'qdamp' and 'qbroad' factor of the instrument in a pipeline process.
-
-    A pipeline to do image background subtraction, auto masking, integration, PDF transformation and PDF
-    modeling to calibrate the qdamp and qbroad. Also, the accuracy of the calibration is tested by the modeling.
-    The output will be the processed data in 'iq', 'sq', 'fq', 'gr' files (depends on 'cfg' file), the fitting
-    results in 'res' file, the refined structure in 'cif' file, the best fits data in 'fgr' file.
-
-    Parameters
-    ----------
-    poni_file : str
-        The path to the poni file. It will be read by pyFAI.
-
-    img_file : str
-        The path to the image file. It will be read by fabio.
-
-    cfg_file : str
-        The path to the PDF configuratino file. Usually, a 'cfg' file or a processed data file like 'gr' file
-        with meta data in the header. If None, use the 'Ni_gr_file.gr' in 'pdfstream/test_data'.
-
-    stru_file : str
-        The path to the structure file. Usually, a 'cif' file. If None, use the 'Ni_cif_file.cif' in
-        'pdfstream/test_data'.
-
-    output_dir : str
-        The directory to save all the outputs. Default current working directory.
-
-    fit_range : tuple
-        The rmin, rmax and rstep in the unit of angstrom.
-
-    qdamp0 : float
-        The initial value for the Q damping factor.
-
-    qbroad0 : float
-        The initial vluae for the Q broadening factor.
-
-    bg_img_file : str
-        The path to the background image file. It should have the same dimension as the data image. If None,
-        no background subtraction will be done.
-
-    bg_scale : float
-        The scale for background subtraction. If None, use 1.
-
-    mask_setting : dict or 'OFF'
-        The settings for the auto-masking. See the arguments for mask_img (
-        https://xpdacq.github.io/xpdtools/xpdtools.html?highlight=mask_img#xpdtools.tools.mask_img). To turn
-        off the auto masking, enter "OFF".
-
-    integ_setting : dict
-        The settings for the integration. See the arguments for integrate1d (
-        https://pyfai.readthedocs.io/en/latest/api/pyFAI.html#module-pyFAI.azimuthalIntegrator).
-
-    img_setting : dict or 'OFF'
-        The keywords for the matplotlib.pyplot.imshow (
-        https://matplotlib.org/3.2.1/api/_as_gen/matplotlib.pyplot.imshow.html). Besides, there is a key
-        'z_score', which determines the range of the colormap. The range is mean +/- z_score * std in the
-        statistics of the image. To turn of the image, enter "OFF".
-
-    chi_plot_setting : dict or 'OFF'
-        The kwargs of chi data plotting. See matplotlib.pyplot.plot(
-        https://matplotlib.org/3.2.1/api/_as_gen/matplotlib.pyplot.plot.html). If 'OFF', skip visualization.
-
-    pdf_plot_setting : dict or 'OFF'
-        The kwargs of pdf data plotting. See matplotlib.pyplot.plot(
-        https://matplotlib.org/3.2.1/api/_as_gen/matplotlib.pyplot.plot.html). If 'OFF', skip visualization.
-
-    ncpu : int
-        The number of cpu used in parallel computing in the modeling. If None, no parallel computing.
-    """
-    if cfg_file is None:
-        cfg_file = resource_filename('pdfstream', 'data/Ni_cfg_file.cfg')
-    if stru_file is None:
-        stru_file = resource_filename('pdfstream', 'data/Ni_cif_file.cif')
-    ai = io.load_ai_from_poni_file(poni_file)
-    img = io.load_img(img_file)
-    pdfconfig = io.load_pdfconfig(cfg_file)
-    stru = io.load_crystal(stru_file)
-    bg_img = io.load_img(bg_img_file) if bg_img_file is not None else None
-    pdfgetter, recipe = calib.calib_pipe(
-        ai, img, pdfconfig, stru, fit_range=fit_range, qdamp0=qdamp0, qbroad0=qbroad0,
-        bg_img=bg_img, bg_scale=bg_scale, mask_setting=mask_setting, integ_setting=integ_setting,
-        chi_plot_setting=chi_plot_setting, img_setting=img_setting, pdf_plot_setting=pdf_plot_setting,
-        ncpu=ncpu
-    )
-    img_path = PurePath(img_file)
-    io.write_out(output_dir, img_path.name, pdfgetter)
-    model.save(recipe, base_name=img_path.name, folder=output_dir)
-    return
