@@ -3,6 +3,7 @@ from pathlib import Path
 import pytest
 from pkg_resources import resource_filename
 
+import pdfstream.servers
 import pdfstream.servers.base as mod
 
 fn = Path(resource_filename("tests", "configs"))
@@ -13,3 +14,9 @@ fn = Path(resource_filename("tests", "configs"))
 )
 def test_find_cfg_file(name):
     mod.find_cfg_file(fn, name)
+
+
+def test_StartStopCallback(simple_stream):
+    cb = pdfstream.servers.base.StartStopCallback()
+    for name, doc in simple_stream:
+        cb(name, doc)
